import unittest

from smufolib.objects.font import Font
from smufolib.pointUtils import (
    Point,
    Position,
    getPoints,
    getContourPoints,
    getCompositePoints
)

# pylint: disable=C0115, C0116, C0103


class PointUtils(unittest.TestCase):

    def setUp(self):
        pass

    def test_getPoints(self):
        pass

    def test_getContourPoints(self):
        pass

    def test_getCompositePoints(self):
        pass


if __name__ == '__main__':
    unittest.main()
