"""where SMuFL meets UFO

"""
from smufolib._version import __version__
from smufolib.objects.engravingDefaults import EngravingDefaults
from smufolib.objects.engravingDefaults import ENGRAVING_DEFAULTS_KEYS
from smufolib.objects.font import Font
from smufolib.objects.glyph import Glyph
from smufolib.objects.layer import Layer
from smufolib.objects.range import Range
from smufolib.objects.smufl import Smufl
from smufolib.objects.smufl import ANCHOR_NAMES
from smufolib.objects.smufl import FONT_ATTRIBUTES
from smufolib.objects.smufl import GLYPH_ATTRIBUTES
from smufolib.request import Request
