import unittest
from smufolib import EngravingDefaults

# pylint: disable=C0115, C0116, C0103


class EngravingDefaults(unittest.TestCase):
    def test_clear(self):
        pass

    def test_items(self):
        pass

    def test_keys(self):
        pass

    def test_update(self):
        pass

    def test_values(self):
        pass

    def test_smufl(self):
        pass

    def test_font(self):
        pass

    def test_glyph(self):
        pass

    def test_layer(self):
        pass

    def test_arrowShaftThickness(self):
        pass

    def test_barlineSeparation(self):
        pass

    def test_beamSpacing(self):
        pass

    def test_beamThickness(self):
        pass

    def test_bracketThickness(self):
        pass

    def test_dashedBarlineDashLength(self):
        pass

    def test_dashedBarlineGapLength(self):
        pass

    def test_dashedBarlineThickness(self):
        pass

    def test_hairpinThickness(self):
        pass

    def test_hBarThickness(self):
        pass

    def test_legerLineExtension(self):
        pass

    def test_legerLineThickness(self):
        pass

    def test_lyricLineThickness(self):
        pass

    def test_octaveLineThickness(self):
        pass

    def test_pedalLineThickness(self):
        pass

    def test_repeatBarlineDotSeparation(self):
        pass

    def test_repeatEndingLineThickness(self):
        pass

    def test_slurEndpointThickness(self):
        pass

    def test_slurMidpointThickness(self):
        pass

    def test_staffLineThickness(self):
        pass

    def test_stemThickness(self):
        pass

    def test_subBracketThickness(self):
        pass

    def test_textFontFamily(self):
        pass

    def test_textEnclosureThickness(self):
        pass

    def test_thickBarlineThickness(self):
        pass

    def test_thinBarlineThickness(self):
        pass

    def test_thinThickBarlineSeparation(self):
        pass

    def test_tieEndpointThickness(self):
        pass

    def test_tieMidpointThickness(self):
        pass

    def test_tieMidpointThickness(self):
        pass

    def test_tupletBracketThickness(self):
        pass

    def test_round(self):
        pass

    def test_spaces(self):
        pass


if __name__ == '__main__':
    unittest.main()
