import unittest
# from smufolib import Smufl

# pylint: disable=C0115, C0116, C0103


class Smufl(unittest.TestCase):

    def test_font(self):
        pass

    def test_glyph(self):
        pass

    def test_layer(self):
        pass

    def test_designSize(self):
        pass

    def test_engravingDefaults(self):
        pass

    def test_sizeRange(self):
        pass

    def test_alternates(self):
        pass

    def test_anchors(self):
        pass

    def test_bBox(self):
        pass

    def test_codepoint(self):
        pass

    def test_componentGlyphs(self):
        pass

    def test_range(self):
        pass

    def test_advanceWidth(self):
        pass

    def test_version(self):
        pass

    def test_classes(self):
        pass

    def test_description(self):
        pass

    def test_name(self):
        pass

    def test_isLigature(self):
        pass

    def test_isMember(self):
        pass

    def test_isOptional(self):
        pass

    def test_isRecommended(self):
        pass

    def test_isSalt(self):
        pass

    def test_isSet(self):
        pass

    def test_round(self):
        pass

    def test_toSpaces(self):
        pass

    def test_toUnits(self):
        pass

    def test_spaces(self):
        pass

    def test_base(self):
        pass

    def test_findGlyph(self):
        pass

    def test_suffix(self):
        pass


if __name__ == '__main__':
    unittest.main()
